@@ -676,9 +676,9 @@ BOOL LASreaderBIL::read_point_default()
         if (fread((void*)&elevation, 4, 1, file) != 1)
         {
 #ifdef _WIN32
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #else
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #endif
           npoints = p_count;
           return FALSE;
@@ -690,9 +690,9 @@ BOOL LASreaderBIL::read_point_default()
         if (fread((void*)&elev, 4, 1, file) != 1)
         {
 #ifdef _WIN32
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #else
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #endif
           npoints = p_count;
           return FALSE;
@@ -708,9 +708,9 @@ BOOL LASreaderBIL::read_point_default()
         if (fread((void*)&elev, 2, 1, file) != 1)
         {
 #ifdef _WIN32
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #else
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #endif
           npoints = p_count;
           return FALSE;
@@ -723,9 +723,9 @@ BOOL LASreaderBIL::read_point_default()
         if (fread((void*)&elev, 2, 1, file) != 1)
         {
 #ifdef _WIN32
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #else
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #endif
           npoints = p_count;
           return FALSE;
@@ -741,9 +741,9 @@ BOOL LASreaderBIL::read_point_default()
         if (fread((void*)rgb, 1, nbands, file) != (U32)nbands)
         {
 #ifdef _WIN32
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #else
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #endif
           npoints = p_count;
           return FALSE;
@@ -756,9 +756,9 @@ BOOL LASreaderBIL::read_point_default()
         if (fread((void*)rgb, 1, nbands, file) != (U32)nbands)
         {
 #ifdef _WIN32
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #else
-          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points\n", row, nrows, col, ncols, p_count);
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %" PRId64 " points\n", row, nrows, col, ncols, p_count);
 #endif
           npoints = p_count;
           return FALSE;
@@ -803,27 +803,27 @@ void LASreaderBIL::close(BOOL close_stream)
   if (overflow_I32_x)
   {
 #ifdef _WIN32
-    REprintf( "WARNING: total of %I64d integer overflows in x\n", overflow_I32_x);
+    REprintf( "WARNING: total of %" PRId64 " integer overflows in x\n", overflow_I32_x);
 #else
-    REprintf( "WARNING: total of %lld integer overflows in x\n", overflow_I32_x);
+    REprintf( "WARNING: total of %" PRId64 " integer overflows in x\n", overflow_I32_x);
 #endif
     overflow_I32_x = 0;
   }
   if (overflow_I32_y)
   {
 #ifdef _WIN32
-    REprintf( "WARNING: total of %I64d integer overflows in y\n", overflow_I32_y);
+    REprintf( "WARNING: total of %" PRId64 " integer overflows in y\n", overflow_I32_y);
 #else
-    REprintf( "WARNING: total of %lld integer overflows in y\n", overflow_I32_y);
+    REprintf( "WARNING: total of %" PRId64 " integer overflows in y\n", overflow_I32_y);
 #endif
     overflow_I32_y = 0;
   }
   if (overflow_I32_z)
   {
 #ifdef _WIN32
-    REprintf( "WARNING: total of %I64d integer overflows in z\n", overflow_I32_z);
+    REprintf( "WARNING: total of %" PRId64 " integer overflows in z\n", overflow_I32_z);
 #else
-    REprintf( "WARNING: total of %lld integer overflows in z\n", overflow_I32_z);
+    REprintf( "WARNING: total of %" PRId64 " integer overflows in z\n", overflow_I32_z);
 #endif
     overflow_I32_z = 0;
   }
